# -*- coding: utf-8 -*-
from __future__ import absolute_import

from xml.etree.ElementTree import ParseError

from six.moves.urllib.parse import urljoin
from datetime import datetime, date
import time
import logging
import json
import xml.etree.ElementTree as ET

from fei_ws.utils import normalize_name
from .base import FEIWSBaseClient
from fei_ws import config
from fei_ws.clients import errors as fei_errors
import requests

logger = logging.getLogger('fei-ws.client')


class FEIWSClient(FEIWSBaseClient):
    def __init__(self, username=None, password=None):
        logger.info('initialize FEI WS Client')
        super(FEIWSClient, self).__init__((1, 2), username, password)

    def find_athlete(self, fei_ids=None, family_name=None, first_name=None,
                     gender_code=None, competing_for=None):
        """
        Find athletes based on a certain search criteria.
        :param fei_ids: A tuple of Athlete FEI IDs you want to search for.
        :param family_name: The athlete's family name. Use % to create a `contain` query
        :param first_name: The athlete's first name. User % to create a `contain` query
        :param gender_code: The athlete gender(None=All, M=Male, F=Female)
        :param competing_for: The NOC code of the athlete's country he is competing for
        :return: An array of Athlete objects.
        """

        fei_ids_array = None
        if fei_ids:
            fei_ids_array = self._ows_factory.ArrayOfString(string=fei_ids)
        response = self.get_organizer_data('findAthlete',
            FEIIDs=fei_ids_array, FamilyName=family_name, FirstName=first_name,
            GenderCode=gender_code, CompetingFor=competing_for)
        results = response.AthleteOC if hasattr(response, 'AthleteOC') else []
        if self.normalize:
            NORMALIZE_RANKING_FIELDS = ('AthleteFamilyName', 'HorseName')
            for athlete in results:
                athlete.FamilyName = self._normalize_name(athlete.FamilyName)
                if hasattr(athlete, 'Rankings') and getattr(athlete.Rankings, 'RankingOC', None):
                    for ranking in athlete.Rankings.RankingOC:
                        ranking.AthleteFamilyName = self._normalize_name(getattr(ranking, 'AthleteFamilyName', None))
                        ranking.HorseName = self._normalize_name(getattr(ranking, 'HorseName', None), roman_nummerals=True)
        return results

    def find_official(self, any_id=None, any_name=None, person_gender=None,
                     admin_nf=None, person_status=None, official_function=None,
                     official_function_discipline=None):
        """
        Find officials based on a certain search criteria.
        :param any_id: Return officials having this ID, licence nr, or vet delegat nr.
        :param any_name: Return officials having this family name, first name, nick name.
            use % to make a `contain` search.
        :param person_gender: Return officials with this gender (None=All, M=Male, F=Female)
        :param admin_nf: Return official that are member of the administrative NF
        :param person_status: Return the official having this status:
                10: Search for all competitors
                1: Search only for active competitors
                0: Search only for inactive competitors
                2: Search only for pending competitors
                3: Search only for cancelled competitors
                9: Search only for suspended competitors.
        :param official_function: Search for officials with specified function
        :param official_function_discipline: Return officials having the specified official
                function for this discipline.
        :return: An array of PersonOfficialOC objects
        """
        response = self.get_organizer_data('findOfficial',
            AnyID=any_id, AnyName=any_name, PersonGender=person_gender, AdminNF=admin_nf,
            PersonStatus=person_status, OfficialFunction=official_function,
            OfficialFunctionDiscipline=official_function_discipline)
        results = response.PersonOfficialOC if hasattr(response, 'PersonOfficialOC') else []
        if self.normalize:
            for official in results:
                official.FamilyName = self._normalize_name(getattr(official, 'FamilyName', None))
        return results

    def find_horse(self, fei_ids=None, name=None, sex_code=None, is_pony=None,
                   athlete_fei_id=None):
        """
        Find horses based on a certain search criteria.
        :param fei_ids: A tuple of Horse FEI IDs you want to search for.
        :param name: The name of the horse. Use % to create a contain query
        :param sex_code: The sex code of the horse (None=All, F=Mare, G=Gelding,
                S=Stallion, M=Male Unknown, U=Unknown).
        :param is_pony: Boolean indicating to search for only ponies.
        :param athlete_fei_id: Search for horses associated with this Athlete.
        :return: A list of Horse objects.
        """
        fei_ids_array = None
        if fei_ids:
            fei_ids_array = self._ows_factory.ArrayOfString(string=fei_ids)
        response = self.get_organizer_data('findHorse',
            FEIIDs=fei_ids_array, Name=name, SexCode=sex_code, IsPony=is_pony,
            AthleteFEIID=athlete_fei_id)
        results = response.HorseOC if hasattr(response, 'HorseOC') else []
        if self.normalize:
            NORMALIZE_FIELDS = ('BirthName', 'CurrentName', 'CommercialName', 'CompleteName', 'ShortenName',
                                'HorseSireName', 'HorseDamName', 'HorseSireOfDamName')
            for horse in results:
                for field in NORMALIZE_FIELDS:
                    setattr(horse, field, self._normalize_name(getattr(horse, field, None), roman_nummerals=True))
                if hasattr(horse, 'Rankings') and getattr(horse.Rankings, 'RankingOC', None):
                    for ranking in horse.Rankings.RankingOC:
                        ranking.AthleteFamilyName = self._normalize_name(getattr(ranking, 'AthleteFamilyName', None))
                        ranking.HorseName = self._normalize_name(getattr(ranking, 'HorseName', None), roman_nummerals=True)
        return results

    def search_horse_trainers(self, horse_fei_code=None, person_fei_id=None, eval_date=None, page_number=1):
        """
        Search for horse trainer records

        :param horse_fei_code: The FEI identifier of the horse filters the records and returns only trainers
        linked to this horse. It maybe null.
        :param person_fei_id: The FEI identifier of the person filters the records and returns only the trainer
        having this ID. It may be null
        :param eval_date: The evaluation date filters the horse trainer records and returns only those whose period
        is active at this evaluation date. It may be null.
        :param page_number: Which page to retrieve. 1 = 1 - 100, 2 = 101 - 200
        :return: returns an array of zero or more HorseTrainer objects.
        """
        response = self.get_organizer_data('searchHorseTrainers',
            HorseFEICode=horse_fei_code, PersonFEIID=person_fei_id, EvalDate=eval_date, PageNumber=page_number)
        results = response.HorseTrainer if hasattr(response, 'HorseTrainer') else []
        if self.normalize:
            for trainer in results:
                trainer.CompleteName = self._normalize_name(getattr(trainer, 'CompleteName', None), roman_nummerals=True)
                trainer.FamilyName = self._normalize_name(getattr(trainer, 'FamilyName', None))
        return results

    def find_event(self, id='', show_date_from=None, show_date_to=None, venue_name='', nf='', discipline_code='',
                   level_code='', is_indoor=None):
        """
        Find FEI events based on a certain search criteria.
        :param id: FEI show/event/competition ID.
        :param show_date_from: Filter shows from this date up.
        :param show_date_to: Filter shows from this date down.
        :param venue_name: The name of the place where the show is held.
        :param nf: Country code of the national federation.
        :param discipline_code: Filter shows based on discipline.
        :param level_code: Filter shows based on level.
        :param is_indoor: Filter indoor / outdoor shows.
        :return: Return value: A list of FEI events
        """
        result = self.get_organizer_data(
            'findEvent',
            ID=id, ShowDateFrom=show_date_from, ShowDateTo=show_date_to, VenueName=venue_name, NF=nf,
            DisciplineCode=discipline_code, LevelCode=level_code, IsIndoor=is_indoor)
        if hasattr(result, 'ShowOC'):
            return result.ShowOC
        return []

    def download_results(self, id):
        """
        Download XML results from the FEI. (You need permission from the FEI
         to download results from an event.
        :param id: FEI event/competition ID.
        :return: A string containing the information in FEI XML.
        """
        result = self._ows_client.service.downloadResults(id)
        self._handle_messages(result)
        return result

    def upload_results(self, result_xml_data):
        """
        Upload results to the FEI. (You need permission from the FEI to
        upload results to the FEI.
        :param result_xml_data: Base64Encoded FEI result file.
        :return:
            FileID = can be used to confirm uploaded results
            Results = String indicating if the uploaded succeeded.
                ERR = An error was found while processing the validation
                MAN = An error was found while processing the mandotory check
                OKW = It is ok for saving, but there are warnings
                OKD = the saving has been done
        """
        return self._ows_client.service.uploadResults(result_xml_data)

    def confirm_upload_results(self, file_id):
        """
        Confirm uploaded results. This is only needed when results returned OK with warnings.
        :param file_id: The FileID return by the uploadResults routine.
        :return: True if the saving has been done.
        """
        return self._ows_client.service.confirmUploadResults(file_id)

    def submit_results(self, id):
        """
        Submit the results to the FEI validation for a given event or a
         given competition.
        :param id: FEI event/competition ID.
        :return: True if the results have been successfully submitted.
        """
        return self._ows_client.service.submitResults(id)

    def get_version(self):
        """
        Return the version string of the Core Application software
        :return: Returns a string containing the [major.minor.revision] number.
        If no return value is recieved or the call times out then the web service is not working
        and should not be used.
        """
        return self.get_common_data('getVersion')

    def get_lookup_date_list(self):
        """
        Retrieve a list of cached information types, and the date each was last modified.
        :return: Returns an array of LookupDate objects.
        Each LookupDate object contains the name of an object type and the most recent date of modification
        for any of the lookup records held in that object type. Client applications should keep these
        lookup records in a local cache to improve performance, and reduce server load.
        For each of these LookupDate objects, there is a corresponding method get<LookupDate.Name>List()
        that retrieves a list of all records of that type.
        """
        result = self.get_common_data('getLookupDateList')
        return result.LookupDate


    def get_country_list(self):
        """
        Retrieve a list of countries.
        :return: Returns an array of Country objects.
        Returns an array of Country objects containing all countries and their corresponding
        names and nationally recognized codes (NOC).
        """
        result = self.get_common_data('getCountryList')
        return result.Country

    def get_dicipline_list(self):
        """
        Retrieve a list of discipline records
        :return: Returns an array of Discipline objects.
        """
        result = self.get_common_data('getDisciplineList')
        return result.Discipline

    def get_issuing_body_list(self):
        """
        Retrieve a list of document issuing bodies.
        :return: Returns an array of DocIssuingBody objects.
        """
        result = self.get_common_data('getDocIssuingBodyList')
        return result.DocIssuingBody

    def get_national_federation_list(self):
        """
        Retrieve a list of National Federations.
        :return: Returns an array of NationalFederation objects.
        """
        result = self.get_common_data('getNationalFederationList')
        return result.NationalFederation

    def get_horse_name_kind_change_list(self):
        """
        Retrieve a list of kind of changes for the name of a horse.
        :return: Returns an array of KindChange objects.
        """
        result = self.get_common_data('getHorseNameKindChangeList')
        return result.KindChange

    def get_document_type_list(self):
        """
        Retrieve a list of extension for the document type.
        :return: Returns an array of DocumentType objects.
        """
        result = self.get_common_data('getDocumentTypeList')
        return result.DocumentType

    def get_language_list(self):
        """
        Retrieve a list of languages used for mailings.
        :return: Returns an array of Language objects.
        """
        result = self.get_common_data('getLanguageList')
        return result.Language

    def get_category_list(self):
        """
        Retrieve a list of categories for an event.
        :return: Returns an array of Category objects containing all
        category values and their corresponding names.
        """
        result = self.get_common_data('getCategoryList')
        return result.Category

    def get_address_name_list(self):
        """
        Retrieve a list of address types.
        :return: Returns an array of AddressName objects containing all addresstypes.
        """
        result = self.get_common_data('getAddressNameList')
        return result.AddressName

    def get_horse_gender_list(self):
        """
        Retrieve a list of horse genders.
        :return: Returns an array of Gender objects containing all horse genders.
        """
        result = self.get_common_data('getHorseGenderList')
        return result.Gender

    def get_horse_fei_id_type_list(self):
        """
        Retrieve a list of horse FEI ID types.
        :return: Returns an array of FEIIDType objects containing all horse FEI ID types.
        """
        result = self.get_common_data('getHorseFEIIDTypeList')
        return result.FEIIDType

    def get_person_gender_list(self):
        """
        Retrieve a list of person genders.
        :return: Returns an array of Gender objects containing all person genders.
        """
        result = self.get_common_data('getPersonGenderList')
        return result.Gender

    def get_person_civility_list(self):
        """
        Retrieve a list of person civilities.
        :return: Returns an array of Person Civility objects containing all person civilities.
        """
        result = self.get_common_data('getPersonCivilityList')
        return result.Civility

    def get_official_function_list(self):
        """
        Retrieve a list of official functions.
        :return: Returns an array of OfficialFunction objects containing all additional functions.
        """
        result = self.get_common_data('getOfficialFunctionList')
        return result.OfficialFunction

    def get_official_status_list(self):
        """
        Retrieve a list of official status.
        :return: Returns an array of OfficialStatus objects containing all additional status.
        """
        result = self.get_common_data('getOfficialStatusList')
        return result.OfficialStatus

    def get_additional_role_list(self):
        """
        Retrieve a list of additional roles.
        :return: Returns an array of AdditionalRole objects containing all additional roles.
        """
        result = self.get_common_data('getAdditionalRoleList')
        return result.AdditionalRole

    def get_message_type_list(self):
        """
        Retrieve a list of error message types and warnings.
        :return: Returns an array of MessageType objects.
        """
        result = self.get_common_data('getMessageTypeList')
        return result.MessageType

    def get_season_list(self, discipline_code='S'):
        """
        Returns the list of World Cup seasons for a given discipline
        (jumping and dressage seasons, currently).
        Those season strings can then be used as input in getLeagueList.
        :param discipline_code: Specify which World Cup seasons will be returned:
            S for World Cup Jumping seasons and D for World Cup Dressage seasons.
        :return: The method returns an array of strings corresponding each
        to a World Cup season of the chosen discipline.
        """
        result = self.get_common_data('getSeasonList', DisciplineCode=discipline_code)
        return result.string

    def get_league_list(self, discipline_code='S', season_code=None):
        """
        Returns a list of Leagues for a given discipline and season
        :param discipline_code: Specify which World Cup leagues will be returned:
            S for World Cup Jumping leagues and D for World Cup Dressage leagues.
        :param season_code: Specify a World Cup season.
            Only the leagues valid during this season will be returned.
        :return: The method returns an array of strings corresponding each to a World Cup
            league of the chosen discipline.
        """
        result = self.get_common_data('getLeagueList', DisciplineCode=discipline_code,
                                      SeasonCode=season_code)
        return result.League


class FEIEntrySystemClient:
    def __init__(self):
        self._base_url = config.FEI_ES_BASE_URL

    def _parse_date(self, value):
        if not value:
            return None
        try:
            return date(*(time.strptime(value, '%Y-%m-%d')[0:3]))
        except ValueError:
            return None

    def _parse_date_time(self, value):
        if not value:
            return None
        try:
            return datetime(*(time.strptime(value, '%Y-%m-%d %H:%M:%S')[0:6]))
        except ValueError:
            return None

    def _parse_athletes(self, root):
        athletes = dict()
        for el in root.iter('{http://www.fei.org/Entry}Athlete'):
            athlete = dict(
                birth_date=self._parse_date(el.get('BirthDate', '')),
                competing_for=el.get('CompetingFor'),
                fei_id=int(el.get('FEIID')),
                family_name=normalize_name(el.get('FamilyName')),
                first_name=el.get('FirstName'),
                NF_id=el.get('NFID'),
                gender=el.get('Gender'),
                time_stamp=int(el.get('TimeStamp', 0))
            )
            contact = el.find('{http://www.fei.org/Entry}ContactPerson')
            if contact:
                athlete['email'] = contact.get('Email')
                athlete['phone'] = contact.get('MobilePhone')
                athlete['contact_name'] = contact.get('Name')
                athlete['contact_quality'] = contact.get('Quality')
            athletes[athlete['fei_id']] = athlete
        return athletes

    def _parse_horses(self, root):
        horses = dict()
        for el in root.iter('{http://www.fei.org/Entry}Horse'):
            horse = dict(
                fei_id=el.get('FEIID'),
                id_type=el.get('IDType'),
                bedding=el.get('Bedding'),
                birt_date=self._parse_date(el.get('BirthDate', '')),
                birth_country=el.get('BirthCountry'),
                breeder=el.get('Breeder'),
                chip_id=el.get('ChipID'),
                color=el.get('Color'),
                color_complement=el.get('ColorComplement'),
                dam=normalize_name(el.get('Dam'), True),
                gender=el.get('Gender'),
                name=normalize_name(el.get('Name'), True),
                owner=normalize_name(el.get('Owner')),
                pony=el.get('Pony', 'false') == 'true',
                sex=el.get('Sex'),
                shorten_name=el.get('ShortenName'),
                sire=normalize_name(el.get('Sire'), True),
                sire_of_dam=normalize_name(el.get('SireOfDam'), True),
                stallion=el.get('Stallion') == 'true',
                studbook=el.get('StudBook'),
                studbook_code=el.get('StudBookCode'),
                NF_id=el.get('NFID'),
                UELN=el.get('UELN'),
                time_stamp=int(el.get('TimeStamp', 0))
            )
            horses[horse['fei_id']] = horse
            horse['ownership'] = self._parse_owners(el)
        return horses

    def _parse_owners(self, root):
        owners = []
        for el in root.iter('{http://www.fei.org/Entry}Owner'):
            owner = dict(
                corperation_name=el.get('CorporationName'),
                family_name=el.get('FamilyName'),
                first_name=el.get('FirstName')
            )
            owners.append(owner)
        return owners


    def _parse_event_entries(self, root, event_code=None):
        events = dict()
        for el in root.iter('{http://www.fei.org/Entry}Event'):
            if event_code and el.get('FEIID', '').lower() != event_code:
                continue  # skip events that are not part of the query

            event = dict(
                fei_id=el.get('FEIID'),
                athlete_max_age=el.get('AthletesMaxAge'),
                athlete_min_age=el.get('AthletesMinAge'),
                category=el.get('Category'),
                category_code=el.get('CategoryCode'),
                code=el.get('Code'),
                definite_entries_date=self._parse_date(el.get('DefiniteEntriesDate', '')),
                discipline=el.get('Discipline'),
                discipline_code=el.get('DisciplineCode'),
                end_date=self._parse_date(el.get('EndDate', '')),
                horse_max_age=el.get('HorsesMaxAge'),
                horse_min_age=el.get('HorsesMinAge'),
                indoor=el.get('Indoor', 'false') == 'true',
                last_date_substitution=self._parse_date(el.get('LastDateSubstitution', '')),
                level=el.get('Level'),
                level_code=el.get('LevelCode'),
                official_team=el.get('OfficialTeam', 'false') == 'true',
                start_date=self._parse_date(el.get('StartDate', '')),
                start_date_subst_addit_OC=self._parse_date(el.get('StartDateSubstAdditOC', '')),
                time_stamp=int(el.get('TimeStamp', 0))
            )
            events[event['fei_id']] = event
            entries = []
            event['entries'] = entries
            for entryEl in el.iter('{http://www.fei.org/Entry}AthleteEntry'):
                entry = self._parse_athlete_entry(entryEl)
                entries.append(entry)
                horses = []
                for horseEl in entryEl.iter('{http://www.fei.org/Entry}HorseEntry'):
                    horses.append(self._parse_entry(horseEl))
                entry['horses'] = horses
        return events

    def _parse_entry(self, el):
        entry = dict(
            acceptation_date=self._parse_date_time(el.get('AcceptationDate', '')),
            accepted=el.get('Accepted') == 'true',
            competitions=el.get('Competitions'),
            definite=el.get('Definite') == 'true',
            definite_entry_date=self._parse_date(el.get('DefiniteEntryDate', '')),
            fei_id=el.get('FEIID'),
            NF_comment=el.get('NFComment'),
            nominated=el.get('Nominated') == 'true',
            nominated_entry_date=self._parse_date_time(el.get('NominatedEntryDate', '')),
            pending=el.get('Pending') == 'true',
            rejected=el.get('Rejected') == 'true',
            rejection_comment=el.get('RejectionComment'),
            rejection_date=self._parse_date_time(el.get('RejectionDate', '')),
            substituted=el.get('Substituted') == 'true',
            withdrawn=el.get('Withdrawn') == 'true',
            time_stamp=int(el.get('TimeStamp', 0))
        )
        return entry

    def _parse_athlete_entry(self, el):
        entry = self._parse_entry(el)
        entry['fei_id'] = int(entry['fei_id'])
        entry['borrowed_horses'] = el.get('BorrowedHorses') == 'true'
        entry['entering_nf'] = el.get('EnteringNF')
        entry['team'] = el.get('Team')
        return entry

    def _parse_show(self, root):
        for el in root.iter('{http://www.fei.org/Entry}ShowEntries'):
            show_xml = el.find('{http://www.fei.org/Entry}Show')
            if not show_xml:
                return None
            show = dict(
                end_date=self._parse_date(show_xml.get('EndDate', '')),
                start_date=self._parse_date(show_xml.get('StartDate', '')),
                fei_id=show_xml.get('FEIID'),
                type=show_xml.get('Type'),
                type_code=show_xml.get('TypeCode'),
                time_stamp=int(show_xml.get('TimeStamp', 0))
            )
            venue_xml = show_xml.find('{http://www.fei.org/Entry}Venue')
            if venue_xml:
                venue = dict(
                    country=venue_xml.get('Country'),
                    country_NOC=venue_xml.get('CountryNOC'),
                    name=venue_xml.get('Name')
                )
                show['venue'] = venue
            return show


    def _parse_root(self, content):
        try:
            return ET.fromstring(content)
        except ParseError:
            return None

    def _parse_entry_file(self, content, event_code=None):
        root = self._parse_root(content)
        if root is None:
            return {}
        show = self._parse_show(root)
        if not show:
            return
        show['athletes'] = self._parse_athletes(root)
        show['horses'] = self._parse_horses(root)
        if event_code:
            show['events'] = self._parse_event_entries(root, event_code.lower())
        else:
            show['events'] = self._parse_event_entries(root)
        return show

    def get_event_entries(self, event_code):
        url = urljoin(self._base_url, 'index.php')
        response = requests.get(url, params={'doc': 'EntryFileDownload', 'event_id': event_code})
        if response.status_code == 200:
            return self._parse_entry_file(response.content, event_code)

    def get_show_entries(self, show_code):
        url = urljoin(self._base_url, 'index.php')
        response = requests.get(url, params={'doc': 'EntryFileDownload', 'show_id': show_code})
        if response.status_code == 200:
            return self._parse_entry_file(response.content)


class FEIEntrySystem3Client:
    def __init__(self, username=None, password=None):
        logger.info('initialize FEI Entry System V3 Client')
        self._username = username if username else config.FEI_WS_USERNAME
        self._password = password if password else config.FEI_WS_PASSWORD
        self._base_url = config.FEI_ESV3_BASE_URL
        self._session = requests.session()
        self._session.headers.update({'Cache-Control': "no-cache", "Content-Type": "application/json"})
        self._authenticate()

    def _authenticate(self):
        if not self._username or not self._password:
            raise fei_errors.FEIWSConfigException('Could not login: username and password are empty.')
        url = urljoin(self._base_url, '/login')
        response = self._session.post(
            url,
            data=json.dumps({'username': self._username, 'password': self._password})
        )

        if response.status_code == 400:  # Bad Request
            data = response.json()
            raise fei_errors.FEIWSApiException(data['message'], 400)
        elif response.status_code == 401:  # Bad Password
            raise fei_errors.FEIWSAuthException('Could not login: PasswordNotMatch', 401)
        elif response.status_code == 200:
            data = response.json()
            self._session.headers.update({'Authorization': "Bearer {}".format(data['token'])})
            return
        raise fei_errors.FEIWSApiException("Could not authenticate", 400)

    def _handle_error(self, response, data=None):
        if response.status_code > 400:
            if data and 'message' in data and 'code' in data:
                raise fei_errors.FEIWSApiException(data['message'], data['code'])
            raise fei_errors.FEIWSApiException('Bad Response', response.status_code)
        return response

    def _handle_json_response(self, response):
        try:
            data = response.json()
        except ValueError as e:
            raise fei_errors.FEIWSApiException('No JSON object could be decoded', 400)
        self._handle_error(response, data)
        return data

    def set_session(self, fei_id, password=None):
        """
        Session route to set or change the FEI user associated to requests. A valid Application token is required.
        :param fei_id: fei_id or FEI User id
        :param password: If set use password to authenticate, else only FEI_ID(NF ONLY)
        :return: A response object
        """
        url = urljoin(self._base_url, '/sessions')
        if not password:
            data = json.dumps({'fei_id': fei_id})
        else:
            data = json.dumps({'username': fei_id, 'password': password})
        response = self._session.post(url, data=data)
        self._handle_error(response)
        if response.status_code == 200:
            self._session.headers.update({'Authorization': response.headers['Authorization']})
        return response

    def set_act_as(self, act_as):
        """
        Route to change the role the current user will act as if the user is allowed to act with multiple roles (FEI, OC, NF, Athlete etc.)
        :param act_as: New role to use. Possible values: “fei”, “oc”, “nf”, “official”, “athlete”, “ath_manager or “groom”
        :return: Response object
        """
        url = urljoin(self._base_url, '/sessions')
        response = self._session.post(url, data=json.dumps({'act_as': act_as}))
        self._handle_error(response)
        if response.status_code == 200:
            self._session.headers.update({'Authorization': response.headers['Authorization']})
        return response

    def get_authorizations(self):
        """
        Returns the entry actions the connected user can perform depending on his permissions.
        :return: Array of Entry action codes
        """
        url = urljoin(self._base_url, '/user/authorizations')
        response = self._session.get(url)
        return self._handle_json_response(response)

    def get_show_entries(self, show_code, **kwargs):
        """
        Return the list of entries of a show for session user
        :param show_code: Show to retrieve.
        :param kwargs: Possible parameters
            <page>: page number.
            <items_per_page>: Items returned per page. Accepted values: 1 to 5000
            <sort>: String to sort results.
            <entries_for_nf>: NF code NOC. Filter entries from this NF.
            <team>: Search entries with corresponding team.
            <athlete_fei_id>: Search entries concerning only the given athlete.
            <horse_fei_id>: Search entries concerning only the given horse. (only for Web and Mobile Apps)
            <no_pagination>: 1 if pagination is not required. The API will returns the number of items_per_page.
            <list_type>: One of EntryListType. (from_invitation_oc, waiting, pending, accepted, rejected, all,
                all_myinvitations, no_show, only_empty_slot, all pr_accepted, pr_minitour_accepted, pr_pending,
                accept_ath, reject_ath, accept_hor, reject_hor, accept_subst_ath, reject_subst_ath, accept_subst_hor,
                reject_subst_hor)
        :return: Array of Entry elements.
        """
        url = urljoin(self._base_url, '/user/shows/{show_code}/entries'.format(show_code=show_code))
        response = self._session.get(url, params=kwargs)
        return self._handle_json_response(response)

    def get_event_entries(self, event_code, **kwargs):
        """
        Returns the list of entries of an event for session user. A valid API Token is required.
        Athlete session users can only request data for their entries.
        NF can receive entries for their NF.
        OC can receive entries with statuses : waiting, accepted, withdrawn, rejected.
        Actions on entries may depend on session user and event status. (Read Only, Consultation mode...).
        :param event_code: Event Code to retrieve.
        :param kwargs: contains the params to include in the url. Possible params:
            <page>: Page number
            <items_per_page>: Items returned per page. Accepted values: 1 to 5000
            <sort>: String to sort results.
            <entries_for_nf>: NF code NOC. Filter entries from this NF.
            <list_type>: One of EntryListType.
            <team>: Search entries with corresponding team.
            <athlete_fei_id>: Search entries concerning only the given athlete.
        :list_types: from_invitation_oc, waiting, pending, accepted, rejected, all,
                all_myinvitations, no_show, only_empty_slot, all pr_accepted, pr_minitour_accepted, pr_pending,
                accept_ath, reject_ath, accept_hor, reject_hor, accept_subst_ath, reject_subst_ath, accept_subst_hor,
                reject_subst_hor
        :return: Array of Entry elements.
        """
        url = urljoin(self._base_url, '/user/events/{event_id}/entries'.format(event_id=event_code))
        response = self._session.get(url, params=kwargs)
        return self._handle_json_response(response)

    def search_athlete(self, event_code, **kwargs):
        """
        Return athlete list for this event and for current session user. Athletes cannot list athletes.
        NF can list only athlete competing for the current NF.
        :param event_code: Event code to retrieve
        :param kwargs: contains the params to include in the url. Possible params:
            <page>: Page number
            <items_per_page>: Items returned per page. Accepted values: 1 to 5000
            <sort>: String to sort results.
            <athlete>: Athlete name or FEI ID.
            <athlete_mode>: Athlete name search mode : [contains|start_with], default is 'start_with'.
            <list_mode>: Infos returned : [wo_checking_entries|all], default is 'all' (`wo_checking_entries` not return entries)
            <registered>: Accepted values: 1 or 0
            <age_limit>: Accepted values: 1 or 0
            <qualified>: Accepted values: 1 or 0
            <no_pagination>: 1 if pagination is not required. The API will returns the number of items_per_page.
            <active_athlete>: Values : active // inactive // all. Default: active. Only returns athlete with the provided state.
            <show_host_athletes>: NF User only, returns hosted athletes or not. Default value = 1 (true).
        :return: Array of “SearchAthlete” objects
        """
        url = urljoin(self._base_url, '/user/events/{event_code}/athletes'.format(event_code=event_code))
        response = self._session.get(url, params=kwargs)
        return self._handle_json_response(response)

    def search_horses(self, event_code, **kwargs):
        """
        Searches horses in the requested event and for current session user. A valid application Token is required.
        Info on blocked horses and registrations may also be returned.
        :param event_code: Event code to retrieve
        :param kwargs: contains the params to include in the url. Possible params:
            <page>: Page number
            <items_per_page>: Items returned per page. Accepted values: 1 to 5000
            <sort>: How to sort. Default is name. Values(name, fei_id).
            <horse>: Athlete name or FEI ID.
            <favorite_horses_athlete>: Returns the horses with imported results for the given athlete FEI ID. Only the last 12 months results are checked.
            <registered>: Accepted values: 1 or 0
            <age_limit>: Accepted values: 1 or 0
            <nf_noc>: NF NOC code or empty for all. Default value is session user’s NF
        :return: Array of “SearchHorse” objects
        """
        url = urljoin(self._base_url, '/user/events/{event_code}/horses'.format(event_code=event_code))
        response = self._session.get(url, params=kwargs)
        return self._handle_json_response(response)


import fei_ws.config as config
import re

NAME_REGEX = re.compile(r"([A-Za-z']\.|\b[A-Za-z']{1,3}\b|'[tT])", re.UNICODE)
ROMAN_REGEX = re.compile(r'\bM{0,4}(CM|CD|D?C{0,3})(XC|XL|L?X{0,3})(IX|IV|V?I{0,3})$', re.IGNORECASE)


def normalize_name(value, roman_numerals=False):
    def _transform(match):
        word = match.group()
        if len(word) == 2 and word[1] == '.':  # If word is a letter followed by a .(dot) Then use capitals
            return word.upper()
        if word.lower() in config.FEI_WS_LOWER_CASE_WORDS:
            return word.lower()
        if word.upper() in config.FEI_WS_UPPER_CASE_ACRONYMS:
            return word.upper()
        return word

    if not value:
        return value
    value = value.title()
    value = NAME_REGEX.sub(_transform, value)
    if roman_numerals:
        value = ROMAN_REGEX.sub(
            lambda match: match.group().upper() if match.group() else match.group(), value)
    return value[0].upper() + value[1:]


import logging

FEI_WS_USERNAME = ''
FEI_WS_PASSWORD = ''
FEI_WS_BASE_URL = 'https://data.fei.org/'
FEI_ESV3_BASE_URL = 'https://es3-api.fei.org/'
FEI_ES_BASE_URL = 'https://entry.fei.org/'
FEI_WS_USE_LOCAL_WSDL = False
FEI_WS_NORMALIZE_NAMES = True
FEI_WS_LOWER_CASE_WORDS = [
    'a', 'an', 'the', 'at', 'by', 'for', 'in', 'of', 'on', 'to', 'up', 'and', 'as', 'but', 'it',
    'or', 'nor', 'de', 'het', 'een', "'t", 'der', 'die', 'das', 'ein', 'des', 'dem', 'den', 'van',
]

FEI_WS_UPPER_CASE_ACRONYMS = [
    'SAP', 'KLM', 'VDL', 'SFN', 'LLC', 'S.A.R.L.'
]

logger = logging.getLogger('fei-ws.client')

try:
    from django.conf import settings
    FEI_WS_BASE_URL = getattr(settings, 'FEI_WS_BASE_URL', FEI_WS_BASE_URL)
    FEI_WS_USERNAME = getattr(settings, 'FEI_WS_USERNAME', FEI_WS_USERNAME)
    FEI_WS_PASSWORD = getattr(settings, 'FEI_WS_PASSWORD', FEI_WS_PASSWORD)
    FEI_ESV3_BASE_URL = getattr(settings, 'FEI_ESV3_BASE_URL', FEI_ESV3_BASE_URL)
    FEI_ES_BASE_URL = getattr(settings, 'FEI_ES_BASE_URL', FEI_ES_BASE_URL)
    FEI_WS_USE_LOCAL_WSDL = getattr(settings, 'FEI_WS_USE_LOCAL_WSDL', FEI_WS_USE_LOCAL_WSDL)
    FEI_WS_NORMALIZE_NAMES = getattr(settings, 'FEI_WS_NORMALIZE_NAMES', FEI_WS_NORMALIZE_NAMES)
    FEI_WS_UPPER_CASE_ACRONYMS = getattr(settings, 'FEI_WS_UPPER_CASE_ACRONYMS', FEI_WS_UPPER_CASE_ACRONYMS)
    FEI_WS_LOWER_CASE_WORDS = getattr(settings, 'FEI_WS_LOWER_CASE_WORDS', FEI_WS_LOWER_CASE_WORDS)
except ImportError as e:
    logger.info('Could not load django.conf')




# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from six.moves.urllib.parse import urljoin

import logging
import os
import re

from zeep.transports import Transport
from zeep.cache import InMemoryCache
from zeep.client import Client
from zeep.exceptions import Fault
from requests import Session

from fei_ws.clients import errors as fei_errors
from fei_ws import config
from fei_ws.utils import normalize_name

logger = logging.getLogger('fei-ws.client')


cache = InMemoryCache(timeout=24*3600*7)


class FEIWSBaseClient(object):
    """FEI Base Client contains logic shared among FEI Clients. It should not
    be used on its own.

    """


    def __init__(self, version, username=None, password=None):
        """ Initializes the base client.

        Params:
            version: A tuple containing the version numbering.
            username: The username used to authenticate. The username from the
                config file is used when it is not supplied.
            password: The password used to authenticate. The password from the
                config file is used when it is not supplied.

        """

        if config.FEI_WS_USE_LOCAL_WSDL:
            __wsdl_path = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'wsdl/')
            self.__AUTH_WSDL = os.path.join(__wsdl_path, 'auth.wsdl')
            self.__ORGANIZER_WSDL = os.path.join(__wsdl_path, 'org_1_2.wsdl')
            self.__COMMON_WSDL = os.path.join(__wsdl_path, 'common.wsdl')
        else:
            self.__AUTH_WSDL = urljoin(config.FEI_WS_BASE_URL,
                                       '/_vti_bin/Authentication.asmx?WSDL')
            self.__ORGANIZER_WSDL = urljoin(
                config.FEI_WS_BASE_URL, '/_vti_bin/FEI/OrganizerWS_%s_%s.asmx?WSDL'
                                        % version)

            self.__COMMON_WSDL = urljoin(config.FEI_WS_BASE_URL,
                                         '/_vti_bin/FEI/CommonWS.asmx?WSDL')

        self.normalize = config.FEI_WS_NORMALIZE_NAMES
        self._version = version
        self._common_data = {}
        self._username = username if username else config.FEI_WS_USERNAME
        self._password = password if password else config.FEI_WS_PASSWORD
        self._session = Session()
        self._ows_client = Client(self.__ORGANIZER_WSDL, transport=Transport(cache=cache, session=self._session))
        self._ows_factory = self._ows_client.type_factory('ns0')
        self._cs_client = Client(self.__COMMON_WSDL, transport=Transport(cache=cache, session=self._session))
        self._cs_factory = self._cs_client.type_factory('ns0')
        self._authenticate([self._cs_client, self._ows_client])
        logger.info('initialize finished')

    def _authenticate(self, clients):
        """Used to authenticate clients with the FEI WS.
        """
        auth_client = Client(self.__AUTH_WSDL, transport=Transport(cache=cache, session=self._session))
        if not self._username or not self._username:
            # Please provide a username and password to init or
            # set the username and password in the settings
            # file.
            raise fei_errors.FEIWSConfigException("Could not login: username and password are empty.")
        auth_login = auth_client.service.Login(self._username, self._password)
        if auth_login['ErrorCode'] != 'NoError':
            raise fei_errors.FEIWSAuthException('Could not login: %s' % auth_login['ErrorCode'])
        logger.info('client authenticated')
        for client in clients:
            client.set_default_soapheaders({'AuthHeader': self._cs_factory.AuthHeader(UserName=self._username, Language='en')})

    def _request(self, service, method, **kwargs):
        try:
            logger.info('Calling service: %s' % method)
            response = getattr(service, method)(**kwargs)
            self._handle_messages(response)
            if hasattr(response.body, '%sResult' % method):
                return getattr(response.body, '%sResult' % method)
            return response.body
        except Fault as ex:
            raise fei_errors.FEIWSApiException(ex.message, ex.code)

    def _handle_messages(self, result):
        """Generic message handler, used to determine if an exception needs to
        be thrown.

        """
        if not 'Messages' in result['body'] or not result['body']['Messages']:
            return
        warnings = ''    
        for message in result['body']['Messages'].Message:
            message_type = fei_errors.FEI_MESSAGE_TYPE_DICT.get(message.UID)
            if not message_type:
                raise fei_errors.FEIWSApiException('Unknown message type: %s' % message.UID, code='MessageTypeUnknown')
            description = ('%s: %s\nDetails: %s' %
                           (message_type['Id'], message_type['Description'], message.Detail))
            if message_type['IsCritical'] or message_type['IsError']:
                logger.exception(description)
                raise fei_errors.FEIWSApiException(message.Detail, code=message_type['Id'])
            warnings += '%s\n' % description
        if warnings:
            logger.warning(warnings)

    def _normalize_name(self, value, roman_nummerals=False):
        return normalize_name(value, roman_nummerals)

    def get_common_data(self, method, **kwargs):
        """Generic method for retrieving data from the common data web service.

        Params:
            method: The name of the common data WS method you want to use.
            **kwargs: Contains the keyword arguments you want to pass to the
                method you are calling.

        Return value: The raw result from the Common Data WS.

        """
        if not kwargs and method in self._common_data:
            return self._common_data[method]
        result = self._request(self._cs_client.service, method, **kwargs)
        self._common_data[method] = result
        return result

    def get_organizer_data(self, method, **kwargs):
        return self._request(self._ows_client.service, method, **kwargs)



from setuptools import setup, find_packages
import fei_ws

setup(
    name='fei-ws',
    version=fei_ws.__version__,
    package_data={
        'fei_ws': ['wsdl/*.wsdl']
    },
    keywords='fei webservice client',
    author='Arend H. Tolner',
    author_email='arie@scgvisual.com',
    packages=find_packages(exclude=['tests']),
    install_requires=['zeep==3.2.0', 'six==1.12.0', 'requests==2.21.0'],
    url='http://www.bounder.nl/',
    license='Mozilla Public License 2.0',
    description='Wrapper for the FEI Web Services v1.2 and FEI Entry System v3',
    long_description=open('README.txt').read(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ]
)



class FEIWSConfigException(Exception):
    pass


class FEIWSAuthException(Exception):
    pass


class FEIWSApiException(Exception):
    def __init__(self, message, code):
        self.code = code
        super(FEIWSApiException, self).__init__(message)


# These message types are generated from the client.get_message_type_list API call.
FEI_MESSAGE_TYPES = [{
    'Id': 'InvalidAddressName',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Address Name does not exist'
}, {
    'Id': 'PermissionDenied',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The connected user is not authorized to perform this operation.'
}, {
    'Id': 'InvalidAdmSBCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "The Studbook DB of the connected user doesn't match the requested Studbook Performances."
}, {
    'Id': 'InvalidCountryCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Country Code is not a valid value'
}, {
    'Id': 'InvalidBirthName',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The BirthName value has an incorrect format'
}, {
    'Id': 'InvalidCommercialName',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Commercial prefix or suffix value has an incorrect format'
}, {
    'Id': 'InvalidCurrentName',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Current Name value has an incorrect format'
}, {
    'Id': 'InvalidDateBirth',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Date of Birth value is too old or in the future.'
}, {
    'Id': 'InvalidDateCastration',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Date of Castration but is incoherent with the current date or Date of Birth or Date of Death.'
}, {
    'Id': 'InvalidDateDeath',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Date of Death value out of range'
}, {
    'Id': 'InvalidDateChronology',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The dates must be in a chronologic order (dateFrom <= dateTo)'
}, {
    'Id': 'InvalidDateRetirement',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Date of Retirement is out of range'
}, {
    'Id': 'InvalidDisciplineCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Discipline Code is not in the Discipline list'
}, {
    'Id': 'InvalidOrganizerNFCountryCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The NF Code for the Organizer NF is not in the National Federation list'
}, {
    'Id': 'NotAllowedOrganizerNFCountryCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'You can use only your own NF NOC code (or null) for the field "OrganizingNFCountryCode"'
}, {
    'Id': 'InvalidRiderCompetingForCountryCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Country Code for the Rider Competing For is not in the Country list'
}, {
    'Id': 'NotAllowedRiderCompetingForCountryCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'You can use only your own NF NOC code (or null) for the field "CompetingForCountryCode"'
}, {
    'Id': 'InvalidFile',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The format of the file is not correct.'
}, {
    'Id': 'InvalidMaxContentLength',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The content of the file is too big.'
}, {
    'Id': 'NotificationError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'An error occurred during the notification process. Please contact your administrator.'
}, {
    'Id': 'InvalidLeagueTypeCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The LeagueTypeCode is not one of the valid option.'
}, {
    'Id': 'InvalidSeasonCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The SeasonCode is not a valid one.'
}, {
    'Id': 'PermissionDeniedChangeStatus',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The change of status is refused for the following reasons, the record is in pending, cancelled or suspended status'
}, {
    'Id': 'InvalidStatus',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The current status of the record do not allow the requested modification'
}, {
    'Id': 'InvalidTicket',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The ticket is invalid or has expired. You must redo the add/update method invocation before reinvoking the confirm method.'
}, {
    'Id': 'PotentialDuplicateFound',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "One or more potential duplicates were found. Your data has not been commited. Please review the record you're trying to insert/update and, if it's adequat, confirm your operation by calling the corresponding confirm method, by providing the received ticket."
}, {
    'Id': 'ConcurrencyUpdateDetected',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Another user has changed these data'
}, {
    'Id': 'RidesDistancesAndSpeedsAreInvalid',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'There must be 2 rides of distances between 40-79 km and 2 rides of distances between 80-90 km at speeds of 16 kph or under.'
}, {
    'Id': 'ConflictWithExistingEntry',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'There is an existing entry related to the Novice Qualification.'
}, {
    'Id': 'DatesAreInvalid',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "The dates should not exceed today's date."
}, {
    'Id': 'HorseNotEndurance',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The horse must have an endurance registration.'
}, {
    'Id': 'FourNationalResults',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'There must be 4 national results.'
}, {
    'Id': 'ThreeQualifyingResultsAtMost',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'There must be three qualifying results at most (but less is possible).'
}, {
    'Id': 'HorseAtLeastEightYearsOldAtLastQualifyingResult',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'There must be three qualifying results at most (but less is possible).'
}, {
    'Id': 'TotalDistanceMustBe240KmAtLeast',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The total distance must be 240 km at least.'
}, {
    'Id': 'TimeSpanBetweenFisrtAndLastQualifyingResultMustBeAtMost36Months',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The time span between the first and the last results must be at most 36 months.'
}, {
    'Id': 'UnexpectedErrorOccured',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'An unexpected error occurred.'
}, {
    'Id': 'PersonNotEndurance',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The person must have an endurance registration.'
}, {
    'Id': 'MissingRoles',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The user does not have the authorization.'
}, {
    'Id': 'ConflictWithExistingNoviceQualification',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Novice Qualification should not have the same qualification start date as an existing one'
}, {
    'Id': 'AllFieldsAreRequired',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'All fields are mandatory.'
}, {
    'Id': 'NoviceQualificationDoesNotExist',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The provided data does not identify an existing Novice Qualification'
}, {
    'Id': 'DistanceIsNotValid',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Distance is not valid.'
}, {
    'Id': 'RankIsNotValid',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Rank is not valid.'
}, {
    'Id': 'AverageSpeedIsNotValid',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Average speed is not valid.'
}, {
    'Id': 'horseIsNotNoviceQualified',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The horse is not Novice Qualified'
}, {
    'Id': 'InvalidTimespanBetweenFirstAndLastResultHorse',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The timespan between the first and the last result must be between 1 year and 2 years'
}, {
    'Id': 'InvalidTimespanBetweenFirstAndLastResultAthlete',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The timespan between the first and the last result must be between 6 months and 2 years'
}, {
    'Id': 'RankPositionIsMandatory',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Rank position is mandatory'
}, {
    'Id': 'AverageSpeedIsMandatory',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Average speed is mandatory'
}, {
    'Id': 'EnduranceTrainerRequired',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'An endurance registered trainer is missing for the specified year'
}, {
    'Id': 'InvalidHeight',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Height of the horse is out of range'
}, {
    'Id': 'InvalidHorseAnyID',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Horse Any ID is too long. See detail for maximum length.'
}, {
    'Id': 'InvalidHorseFEICode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Horse FEI Code is not valid'
}, {
    'Id': 'InvalidHorseNF',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Horse is not administered by your NF'
}, {
    'Id': 'InvalidHorseNFCountry',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Upgrading to Passport is not possible for UE Horses - reference to the Veterinarian Rules, see in https://inside.fei.org/system/files/Letter-EU-NFs_May2011.pdf '
}, {
    'Id': 'InvalidDocGenderCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The value given for DocumentGenderCode is not valid'
}, {
    'Id': 'InvalidHorseIdentity',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The value given for HorseIdentityTypeCode is not valid'
}, {
    'Id': 'NoDocumentLine',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The type of document identified HorseIdentityTypeCode is not currently requested for this horse; check which document is requested'
}, {
    'Id': 'DocPartDoesNotMatchDocGender',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The value given for DocumentGenderCode is not compatible with the value given for HorseIdentityTypeCode'
}, {
    'Id': 'InvalidHorseFeiIDType',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Not a valid Horse Fei ID Type'
}, {
    'Id': 'InvalidIsCastrated',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The IsCastrated value is incompatible with the date of castration'
}, {
    'Id': 'InvalidPonyHeight',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The IsPony value has been set true, but the height is too tall for a pony'
}, {
    'Id': 'InvalidReasonNF',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Excepting for changes on birth name or shorten name, the changes in names are invoiced.'
}, {
    'Id': 'InvalidRequest',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The request is not valid as you are already administering this horse or person. '
}, {
    'Id': 'InvalidShortName',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The ShortName value has an incorrect format'
}, {
    'Id': 'InvalidYear',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Year number is out of range'
}, {
    'Id': 'InvalidRecognitionCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The RecognitionCode must be empty'
}, {
    'Id': 'InvalidRecognitionCodeRO',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The recognition code cannot be changed.'
}, {
    'Id': 'InvalidDateIssuingOutOfRange',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Issuing Date is out of range (01.01.1990 and 31.12.2006)'
}, {
    'Id': 'InvalidDateIssuing',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The DateIssuing is not setted between the DateBirth and today'
}, {
    'Id': 'InvalidIssuingBodyCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The IssuingBodyCode cannot be empty.'
}, {
    'Id': 'InvalidIssuingBodyCodeRO',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The IssuingBodyCode cannot be changed.'
}, {
    'Id': 'InvalidRequestOneIsInPending',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'A request is already in pending status'
}, {
    'Id': 'InvalidRegOnlyWithOldFEIID',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'You cannot add an existing horse with only a registration number'
}, {
    'Id': 'InvalidShortNameRequiredFromCompleteName',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'You must give a shorten name as the complete name longer than 20 characters'
}, {
    'Id': 'InvalidOldFEICode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The old FEI ID has an invalid format.'
}, {
    'Id': 'InvalidOldFEICodeExisting',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The given old FEI ID is already in use in the system.'
}, {
    'Id': 'InvalidUpgradeOperation',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Only horses without a passport or a recognition card can be upgraded.'
}, {
    'Id': 'InvalidDocType',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The document extension is not allowed.'
}, {
    'Id': 'InvalidAlreadyUploaded',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'You cannot upload a diagram as the horse already has a diagram.'
}, {
    'Id': 'HorseAlreadyRegistered',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The horse is already registered in this discipline'
}, {
    'Id': 'InvalidOwnerBeginDateBeforeBirthday',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The OwnerBeginDate must not be before the birthDate'
}, {
    'Id': 'InvalidOwnerBeginDateInFuture',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The OwnerBeginDate must not be in the future'
}, {
    'Id': 'InvalidOwnerBeginDateBeforeLastOwner',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The OwnerBeginDate must not be before the OwnerBeginDate of the previous owner'
}, {
    'Id': 'InvalidUELNPattern',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The pattern of the UELN is incorrect'
}, {
    'Id': 'UELNUniquenessViolation',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The UELN is a unique number, your horse cannot have the same UELN as another horse. Please contact the FEI'
}, {
    'Id': 'ChipIDUniquenessViolation',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The chip ID should be unique.'
}, {
    'Id': 'ObsoleteFunction',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Acces denied'
}, {
    'Id': 'MicrochipIsMandatory',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The microchip number is mandatory if the horse has been created after the 01/01/2013'
}, {
    'Id': 'MicrochipIsAlreadySet',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Microchip value cannot be changed anymore once a value has been provided. Please fill the form at https://inside.fei.org/fei/your-role/veterinarians/passports/microchips and return it to the FEI.'
}, {
    'Id': 'MicrochipIsAlreadySetNoUrl',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Microchip value cannot be changed anymore once a value has been provided.'
}, {
    'Id': 'MicrochipId',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'MicrochipId'
}, {
    'Id': 'InvalidStickerType',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The sticker type is not supported.'
}, {
    'Id': 'NoSuchStickerAvailable',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'No such sticker available for this horse.'
}, {
    'Id': 'NoLongerAvailable',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The limited period to download an original sticker is expired therefore it cannot anymore be downloaded.'
}, {
    'Id': 'InvalidSexCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The provided SexCode is not among the available values.'
}, {
    'Id': 'InvalidTrainerPeriod_2',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The new starting date must be greater than the starting date of the actual horse trainer'
}, {
    'Id': 'InvalidTrainerPeriod_3',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The last day of horse registration year is undefined'
}, {
    'Id': 'InvalidTrainerPeriod_4',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The new start date must be the next day after the last horse trainer end date'
}, {
    'Id': 'InvalidTrainerPeriod_5',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The new end date cannot be greater than the end of the latest horse registration year'
}, {
    'Id': 'InvalidTrainerPeriod_6',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The new end date cannot be smaller than the end of the current registration year'
}, {
    'Id': 'InvalidTrainerPeriod_7',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'This person cannot be chosen as a trainer'
}, {
    'Id': 'GetHorseOwnerWSDisabled',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The method getHorseOwner is deprecated. Use getHorseOwnership instead.'
}, {
    'Id': 'InvalidStudbookCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The studbook code is not valid'
}, {
    'Id': 'AssignHorseOwnerWSDisabled',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The method assignHorseOwner is deprecated. Use assignHorseOwnership instead.'
}, {
    'Id': 'AddOldOwnerFiedsDisabledUseOwnership',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Horse.Ownership should be used to create an owner. Other owner fields in Horse are deprecated.'
}, {
    'Id': 'TotalOwnershipPercentageMustBe100',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'If the ownership percentages are filled, their total must be of 100%'
}, {
    'Id': 'AllPercentageMustBeProvided',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'If at least one percentage is filled, all percentages must be filled.'
}, {
    'Id': 'InvalidDateFrom',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'DateFrom must be greater than the last Since owner date and smaller or equal than today.'
}, {
    'Id': 'HorseNotFound',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Horse not found.'
}, {
    'Id': 'MemberNotFound',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Member (Corporation/Person) not found.'
}, {
    'Id': 'PercentageMustBeEmpty',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Percentage must be empty or different of 0.'
}, {
    'Id': 'HorseHasBeenMigratedToNewOwnershipFormat',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'This method cannot be called on a horse that has an ownership in the new format. Please use the assignHorseOwnership method instead.'
}, {
    'Id': 'UpdateForbiddenSire',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The update of the sire (Name or UELN) of this horse is locked, because a pedigree horse has been linked to it.'
}, {
    'Id': 'UpdateForbiddenDam',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The update of the dam (Name or UELN) of this horse is locked, because a pedigree horse has been linked to it.'
}, {
    'Id': 'UpdateForbiddenSireOfDam',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The update of the sire of dam (Name or UELN) of this horse is locked, because a pedigree horse has been linked to it.'
}, {
    'Id': 'InvalidKindOfName',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Invalid Kind Of Name'
}, {
    'Id': 'BirthNameMustMatchCurrentName',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Birth Name must match Current Name'
}, {
    'Id': 'BirthNameMustDifferCurrentNameOrCommercialNameMustBeSet',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Birth Name must differ from Current Name or Commercial Name must be provided'
}, {
    'Id': 'InvalidBirthDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Birth date is not valid, see linked pedigree horses.'
}, {
    'Id': 'InvalidGender',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Gender is not valid, see linked pedigree horses.'
}, {
    'Id': 'InvalidOwnershipNationality',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'NationalityOfOwnership must be one of the Nationality of the Owners'
}, {
    'Id': 'InvalidMemberStatus',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'At least one member status is invalid.'
}, {
    'Id': 'InvalidMemberAddress',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'At least one member address is invalid.'
}, {
    'Id': 'InvalidAddressAlreadyExist',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Address already exists'
}, {
    'Id': 'OwnershipAddress',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'As this address is used in at least one horse ownership, you cannot change it. If you need to make a correction of that address, please contact FEI.'
}, {
    'Id': 'InvalidBusinessEmailAddress',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The BusinessEmailAddress is not correctly formatted'
}, {
    'Id': 'InvalidHomepage',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Homepage URL is not correctly formatted'
}, {
    'Id': 'InvalidIsActive',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The IsActive cannot be false because a date of death or date of retirement is given.'
}, {
    'Id': 'InvalidIsMailingAddress',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The IsMailingAddress can not be modified'
}, {
    'Id': 'MissingMailingAddress',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The MailingAddress must be set'
}, {
    'Id': 'InvalidMailingLanguage',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Mailing Language value is not valid'
}, {
    'Id': 'MissingNationalities',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'One or more nationality is required'
}, {
    'Id': 'MissingDateOfBirth',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Birth date is mandatory if the person is an Athlete, a Trainer or an Official'
}, {
    'Id': 'InvalidGroupCodes',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Only the group codes given in the Detail property of this message can be modified by the NF'
}, {
    'Id': 'ChangeGroupCodeError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'You are not authorized to change the group codes given in the Detail property'
}, {
    'Id': 'CompetingForLocked',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'CompetingFor is locked.'
}, {
    'Id': 'CompetingForMustBeEmpty',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'You should not specify the CompetingFor if you are not creating an Athlete record.'
}, {
    'Id': 'InvalidEventingCategory',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'InvalidEventingCategory'
}, {
    'Id': 'InvalidPersonAnyID',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Person Any ID is too long. See detail for maximum length.'
}, {
    'Id': 'InvalidPersonFEIID',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The PersonFEIID is not valid'
}, {
    'Id': 'InvalidNFNoc',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The NF NOC is not valid'
}, {
    'Id': 'InvalidAddressPerson',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Person Address Name does not exist'
}, {
    'Id': 'InvalidPrivateEmailAddress',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The PrivateEmailAddress is not correctly formatted'
}, {
    'Id': 'EmailLockedByUserStatus',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The email is locked due to the user status (the person has a pending, active or suspended user account).'
}, {
    'Id': 'PersonAlreadyRegistered',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The person is already registered in this discipline'
}, {
    'Id': 'MissingLeague',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "The WC person's league must be set first"
}, {
    'Id': 'LeagueAlreadySet',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "The WC person's league is already set in this discipline"
}, {
    'Id': 'OnlyOneLeague',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "The WC person's league can be set only if several natural leagues exist in this discipline"
}, {
    'Id': 'NoLeague',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "No WC person's league defined"
}, {
    'Id': 'InvalidLeague',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'This league is unknown in this discipline'
}, {
    'Id': 'InvalidGenderCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The provided GenderCode is not among the available values.'
}, {
    'Id': 'InvalidNationalities',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'At least one nationality is invalid'
}, {
    'Id': 'GroupCodesNotFound',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The value in GroupCodes contains at least one invalid code'
}, {
    'Id': 'PersonUpgradeNotAllowed',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'PersonUpgradeNotAllowed'
}, {
    'Id': 'TrainerAlreadyRegistered',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The trainer is already registered'
}, {
    'Id': 'OwnerOwnsAHorse',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The owner group cannot be removed since the person is owning a horse.'
}, {
    'Id': 'personIsNotNoviceQualified',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The person is not Novice Qualified'
}, {
    'Id': 'CorporationContactIsContact',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The corporation contact group cannot be removed since the person is currently person of contact for a corporation.'
}, {
    'Id': 'NotAllowedToRemoveANationality',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'It is not allowed to remove a nationality from a person. If a nationality is wrong, please contact the FEI.'
}, {
    'Id': 'DuplicateNationality',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Duplicate nationality was found.'
}, {
    'Id': 'InvalidCorporationFEIID',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'This corporation FEIID is not valid'
}, {
    'Id': 'InvalidCorporationAddress',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'At least one address is invalid.'
}, {
    'Id': 'InvalidAddressCountry',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'At least one address has an invalid country.'
}, {
    'Id': 'InvalidShowCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The provided ShowCode does not exist'
}, {
    'Id': 'InvalidShowMinDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Start Date cannot be smaller than the 01.01.1900'
}, {
    'Id': 'InvalidShowDateYear',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The end date and the start date must be in the same year'
}, {
    'Id': 'InvalidShowDateDeadLine',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The deadline is expired'
}, {
    'Id': 'InvalidShowDuration',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The show duration is too long'
}, {
    'Id': 'InvalidVenue',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The venue_code is invalid or a venue not authorized for the NF members'
}, {
    'Id': 'PermissionDeniedShowTypeToModify',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The only Standard show  type (CI) is alllowed to modify by the NF members'
}, {
    'Id': 'InvalidStartEventDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The StartEventDate must to be inside the showDate'
}, {
    'Id': 'InvalidEndEventDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The EndEventDate must to be inside the showDate'
}, {
    'Id': 'InvalidIndoorOudoor',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The choice of the Indoor/Outdoor is invalid for this event type'
}, {
    'Id': 'InvalidEventTypeCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'This EventTypeCode does not exist or not permitted.'
}, {
    'Id': 'LockedIsEventApprovedByFEI',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Locked because there is inside the show at the least a event approved by FEI'
}, {
    'Id': 'LockedIsCompetition',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Locked because there is at the least a competition inside a event'
}, {
    'Id': 'PermissionShowEventDenied',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Impossible to modify the show/event for one or more of the following reasons: the show/event was deleted, cancelled, approved by the FEI, contains competitions or the period for modification is expired for that year.'
}, {
    'Id': 'DraftScheduleImportError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Error while importing Draft Schedule document'
}, {
    'Id': 'InvalidEntriesInPrincipleDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Invalid EntriesInPrincipleDate. This date must be less or equal to the event start date.'
}, {
    'Id': 'InvalidNominatedEntriesDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Invalid InvalidNominatedEntriesDate. This date must be less or equal to the event start date.'
}, {
    'Id': 'InvalidDefiniteEntriesDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Invalid InvalidDefiniteEntriesDate. This date must be less or equal to the event start date.'
}, {
    'Id': 'InvalidSubstitutionsLastDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Invalid InvalidSubstitutionsLastDate. This date must be less or equal to the event start date.'
}, {
    'Id': 'InvalidSubstitutionsByOCFirstDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Invalid SubstitutionsByOCFirstDate. This date must be less or equal to the event start date.'
}, {
    'Id': 'InvalidHorseInspectionDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Invalid InvalidHorseInspectionDate. This date must be less or equal to the event start date.'
}, {
    'Id': 'DraftScheduleOnInsert',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "Draft Schedule fields can't be set on creation"
}, {
    'Id': 'InsertAdminLevelError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "Admin Level can't be inserted, this value will be automatically calculated"
}, {
    'Id': 'InsertEntriesInPrincipleDateError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "EntriesInPrincipleDate value can't be set on event creation"
}, {
    'Id': 'InsertNominatedEntriesDateError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "NominatedEntriesDate value can't be set on event creation"
}, {
    'Id': 'InsertDefiniteEntriesDateError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "DefiniteEntriesDate value can't be set on event creation"
}, {
    'Id': 'InsertSubstitutionsLastDateError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "SubstitutionsLastDate value can't be set on event creation"
}, {
    'Id': 'InsertSubstitutionsByOCFirstDateError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "InsertSubstitutionsByOCFirstDateError value can't be set on event creation"
}, {
    'Id': 'InsertHorseInspectionDateError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "HorseInspectionDate value can't be set on event creation"
}, {
    'Id': 'InsertOpenForEntriesError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "OpenForEntries value can't be set on event creation"
}, {
    'Id': 'InvalidAdminLevelFormat',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The AdminLevel format is not valid'
}, {
    'Id': 'PermissionEntriesDatesDenied',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Impossible to modifiy the show/event. You have not sufficient rights to modify Entries Dates.'
}, {
    'Id': 'PermissionOpenEntriesDenied',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Impossible to modifiy the show/event. You have not sufficient rights to modify OpenForEntries'
}, {
    'Id': 'PermissionAdminLevel',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Impossible to modifiy the show/event. You have not sufficient rights to modify AdminLevel.'
}, {
    'Id': 'InvalidCompetitionCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The CompetitionCode is not valid'
}, {
    'Id': 'ResultImportFatal',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Fatal error while importing result data'
}, {
    'Id': 'InvalidEventCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Event Code is not valid'
}, {
    'Id': 'InvalidEventOrCompetitionCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The code provided is not a valid Event Code or a valid Competition Code'
}, {
    'Id': 'NoResult',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'No results in this event'
}, {
    'Id': 'NotImported',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'All results of this event must be imported'
}, {
    'Id': 'LockedByStatus',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'This action is locked by the current result status of the event'
}, {
    'Id': 'InvalidOfficial',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The person is not an official'
}, {
    'Id': 'InvalidOffContactEndDate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The end date cannot be earlier than the start date'
}, {
    'Id': 'InvalidOffContactDisciplineCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The chosen additional role/discipline combination is not valid'
}, {
    'Id': 'InvalidOfficialContactType',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Invalid Official Contact Type'
}, {
    'Id': 'OfficialContactHasNoCertificate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Official Contact has no certificate associated.'
}, {
    'Id': 'InvalidPETStatus',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'There is no accepted PET application for this person'
}, {
    'Id': 'MissingEmailOrNoEmailConfirmationFlag',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The e-mail is Mandatory for an athlete, or the NoEmailConfirmation flag must be set to true'
}, {
    'Id': 'MissingBirthCountryCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The BirthCountryCode is mandatory if the horse has a Passport or a Recognition Card but no breed '
}, {
    'Id': 'MissingColorComplement',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The ColorComplement value is mandatory for this color'
}, {
    'Id': 'MissingConditions',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Conditions parameter is null or empty'
}, {
    'Id': 'MissingDateBirth',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The DateBirth is missing'
}, {
    'Id': 'MissingHorseAnyID',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Horse Any ID is empty or null'
}, {
    'Id': 'MissingHorseDiagram',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'This horse has no diagram'
}, {
    'Id': 'MissingPonyHeight',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Height is mandatory for the Pony'
}, {
    'Id': 'MissingBreed',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Breed is mandatory when the horse has a Passport or a Recognition Card but no country of birth'
}, {
    'Id': 'MissingRecognitionCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The RecognitionCode is required'
}, {
    'Id': 'MissingFEICode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The FEICode is required for horses already having a passport or a recognition card'
}, {
    'Id': 'MissingIssuingBodyCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The IssuingBodyCode is required'
}, {
    'Id': 'MissingPersonAnyID',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Person Any ID is empty or null'
}, {
    'Id': 'MissingCompetingFor',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Competing For is required'
}, {
    'Id': 'MissingEventCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Event Code is empty or null'
}, {
    'Id': 'MissingDisciplineCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Discipline Code is missing'
}, {
    'Id': 'MissingRiderCountryCodeOrOrganizingNFCountryCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Either the "OrganizingNFCountryCode" or the "AthleteCountryCode" fields must be provided'
}, {
    'Id': 'RegistrationTooEarly',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'A registration may not be made more than a few weeks in advance. Currently this is set at 6 weekds, but this is subject to review.'
}, {
    'Id': 'RegistrationTooLate',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The time limit for registration this year is expired.'
}, {
    'Id': 'RegistrationImpossibleButEmailUpdated',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The email was successfully updated but the registration could not made (see other messages).'
}, {
    'Id': 'NoOwner',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The horse does not have an owner'
}, {
    'Id': 'PermissionRequired_NFResult',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The NF Result Admin or NF Result Consult permission is required'
}, {
    'Id': 'NoData',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'There were no Registration records supplied'
}, {
    'Id': 'Locked',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'A field (see description) cannot be modified.'
}, {
    'Id': 'Mandatory',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'A field (see description) is mandatory.'
}, {
    'Id': 'MaxLength',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The field (see description for field name and length) is too long.'
}, {
    'Id': 'BadCriteria',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The search criteria is not recognized or its range is invalid. See detail for the name of the criteria.'
}, {
    'Id': 'BadCode',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': "A field (see description) doesn't contain an recognized value."
}, {
    'Id': 'DownloadLimitReached',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'To much data to download. Please refine your search. See detail for maximum limit.'
}, {
    'Id': 'MissingMandatoryInformation',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Some mandatory information is missing on the record. You have to correct it first.\n    \n    For Horse records, the message is raised either because no owner has been provided or because one or more of the following fields is empty: BirthName, CurrentName, ColorCode, GenderCode, NatPassport, IssuingNFCode. \n    If the horse has a FEI Passport or a Recognition Card then this message is also raised if the Breed or BirthCountryCode field is empty.\n    \n    For Person records, the message is raised when one or more of the following fields is empty: FirstName, FamilyName, NationalityCode, DateOfBirth.'
}, {
    'Id': 'invalidEmailAddress',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Invalid Email Address'
}, {
    'Id': 'InvalidCompetingFor',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The Competing For must be a selected Nationalities'
}, {
    'Id': 'InvalidCompetingForNF',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'The selected CompetingFor does not match an existing NF.'
}, {
    'Id': 'CollectionEmpty',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'the collection is empty'
}, {
    'Id': 'ValidationError',
    'IsCritical': False,
    'IsError': True,
    'IsWarning': False,
    'CEW': 'E',
    'Description': 'Validation error'
}, {
    'Id': 'ListTruncated',
    'IsCritical': False,
    'IsError': False,
    'IsWarning': True,
    'CEW': 'W',
    'Description': 'The returned list was truncated'
}, {
    'Id': 'ResultImportWarning',
    'IsCritical': False,
    'IsError': False,
    'IsWarning': True,
    'CEW': 'W',
    'Description': 'Warning while importing result data'
}, {
    'Id': 'DuplicatedCriteria',
    'IsCritical': False,
    'IsError': False,
    'IsWarning': True,
    'CEW': 'W',
    'Description': 'A criteria is duplicated, only the last given value is used.'
}, {
    'Id': 'NotMapped',
    'IsCritical': False,
    'IsError': False,
    'IsWarning': True,
    'CEW': 'W',
    'Description': 'A field (see description) is not mapped and will be ignored. Do not set this field.'
}, {
    'Id': 'ExportWarning',
    'IsCritical': False,
    'IsError': False,
    'IsWarning': True,
    'CEW': 'W',
    'Description': 'A warning was raised during export. Please see detail for full description.'
}, {
    'Id': 'CriticalException',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'An unexpected error occurred at the server beyond the control of the application. Try again later.'
}, {
    'Id': 'InvalidUser',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'The user is not correct.'
}, {
    'Id': 'UserNotConnected',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'The user is no longer connected. Please connect again'
}, {
    'Id': 'InvalidRoleTypeCode',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'Invalid Role Type'
}, {
    'Id': 'InvalidConnectionTypeCode',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'Invalid connection type code'
}, {
    'Id': 'MissingEmailAddress',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'Missing email address'
}, {
    'Id': 'AccountSuspended',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'AccountSuspended'
}, {
    'Id': 'DBException',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'DB Exception : {0}'
}, {
    'Id': 'DuplicatedRole',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'An identic active role already exists for the invitee.'
}, {
    'Id': 'PersonIsNotOutsider',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'The groups are not those of an outsider.'
}, {
    'Id': 'DuplicatedEmail',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'The email {0} is already in use.'
}, {
    'Id': 'MissingRankingCode',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'Ranking code is mandatory'
}, {
    'Id': 'MissingRankingValidDate',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'Ranking valid date is mandatory'
}, {
    'Id': 'MissingLeagueCode',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'League code is mandatory for this Ranking code'
}, {
    'Id': 'InvalidLeagueCode',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': "This league code doesn't exist or is not suitable for this ranking"
}, {
    'Id': 'InvalidRankingParameter',
    'IsCritical': True,
    'IsError': False,
    'IsWarning': False,
    'CEW': 'C',
    'Description': 'No ranking has found with these parameters'
}]

FEI_MESSAGE_TYPE_DICT = {x['Id']: x for x in FEI_MESSAGE_TYPES}